package model;

/**
 * Estende la classe <code>CadUniverse</code> rendendola adatta a rappresentare l'insieme degli 
 * agenti e dei TupleCenter presenti nel nodo specificato.
 * Il metodo  <code>addAgent</code>  utilizzato per inserire nell'ambiente grafico un agente rappresentato da
 * una sfera.
 * Il metodo  <code>addTC</code>  utilizzato per inserire nell'ambiente grafico un agente rappresentato da
 * un cubo.
 * Il metodo  <code>deleteAgent</code>  utilizzato per eliminare nell'ambiente grafico l'agente specifico
 * Il metodo  <code>deleteTC</code>  utilizzato per eliminare nell'ambiente grafico il tuple center specifico
 * Il metodo  <code>colorAgent</code>  utilizzato per modificare il colore dell'agente specifico
 * Il metodo  <code>colorTC</code>  utilizzato per modificare il colore del tuple center specifico
 * Il metodo  <code>trasparenceTC</code> e <code>trasparenceAgent</code> sono utilizzati per 
 * modificare la trasparenza dei tuple center e degli agenti
 * Il metodo  <code>colorTC</code>  utilizzato per modificare il colore del tuple center specifico
 * Il costruttore inserisce tanti Agenti e Nodi quanti sono i nomi nelle rispettive liste passati come parametri 
 * del costruttore
 * 
 * @author Matteo Campana nmatr.0000197228
 */
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import javax.swing.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.image.TextureLoader;
public class AgTCUniverse extends CadUniverse {
	//max size indica il max numero di nodi presenti nell'ambiente
	//serve per posizionare spazialmente un eventuale nuovo nodo in una posizione nella quale si 
	//sicuri non sia mai stato inserito un nodo...L'approccio  un p laborioso ma evita spiacevoli sovraposizioni
	//Viene 'rinfrescata' se si lancia il metodo initialize
	protected int maxSizeAgent;
	protected int maxSizeTC;
	//Creo due liste locali per differenziare i TC dagli agenti
	//la classe cad universe ha infatti una sola lista dove iserire oggetti grafici
	protected ArrayList aList = new ArrayList();
	protected ArrayList tcList = new ArrayList();
	
	protected JTextField elemSField;
	
	/**
	 * il costruttore inserisce i TupleCenter e gli agenti presenti attualmente nell'ambiente grafico
	 * distribuendoli concentricamente
	 * 
	 * @param agentList La lista degli agenti presenti attualmente
	 * @param tuplecenterList La lista dei tuple center presenti attualmente
 	 * @param nodeSelField Riceve il JTextField sul quale verr visualizzato il nome dell'oggetto selezionato con il mouse
 	 * 
	*/		
	public  AgTCUniverse(List agentList, List tuplecenterList, JTextField elemSelField){
		
		super();
		
		//CREO LO SFONDO IN BACKGROUD COME UNA SFERA DOTATA DI UNA TEXTURE IN TONALIT GRIGIA
		 //Crea un nodo di sfondo
		 Background back=new Background();
		 //Imposta un raggio d'azione per lo sfondo
		 back.setApplicationBounds(new BoundingSphere(new Point3d(0,0,0),2000.0));
		 // create an appearance for the Sphere
		 Appearance app=new Appearance();
		 //Carica una texture
		 Texture texture=new TextureLoader("floor.gif", super.getSimUniverse().getCanvas()).getTexture();
		 //Aggiunge la texture all'aspetto
		 app.setTexture(texture);
		 //Crea una sfera che rappresenter il nostro mondo
		 Sphere sphere=new Sphere(1.0f,Primitive.GENERATE_TEXTURE_COORDS|Primitive.GENERATE_NORMALS_INWARD,app);
		 //Crea un BranchGroup che conter la sfera
		 BranchGroup bgGeometry=new BranchGroup();
		 //Aggiunge la sfera al gruppo
		 bgGeometry.addChild(sphere);
		 //Aggiunge il gruppo allo sfondo
		 back.setGeometry(bgGeometry);
		 //Crea un gruppo per lo sfondo
		 BranchGroup backgroundGroup=new BranchGroup();
		 //Aggiunge lo sfondo al gruppo
		 backgroundGroup.addChild(back);
		 super.BGambiente.addChild(backgroundGroup);
		 
		//INIZIALIZZO LA SCENA
		 elemSField=elemSelField;
		 this.initialize(agentList, tuplecenterList);
	}
	
	/**
	 * Inserisce un Agente nell'universo visualizzandolo come una sfera di colore iniziale bianco
	 * @param nome Il nome da associare all'Agente
	*/		
	public void addAgent(String nome){
		
		Appearance app=new Appearance();
		//Assegna un Materiale all'aspetto
		Color3f color=new Color3f(1.0f,1.0f,1.0f);
		Color3f black=new Color3f(0f,0f,0f);
		app.setMaterial(new Material(color,black,color,black,80.0f));
		//Applico l'atialiasing
		LineAttributes lattr= new LineAttributes();
		lattr.setLineAntialiasingEnable(true);
		app.setLineAttributes(lattr);

		try {
			//Aggiungo una sfera all'univeso rappresentante l'agente
			super.addUserObject(nome,new Sfera(super.getSimUniverse().getCanvas(), 0.5f, app, nome, elemSField));
			//aggiungo il nome anche nella lista locale
			aList.add(nome);
			//aggiorno la maxsize
			maxSizeAgent=maxSizeAgent+1;
			
		} catch (CadObjectNameException e) {
			System.out.println("Nome Agente non valido"+nome);		
		}
		//Lo pongo nella posizione pari a maxSize
		Transform3D tra=new Transform3D();
		Vector3d vett = new Vector3d(0,0,0);
		//Calcolo la posizione dovemetterlo
		LayOutMaker calcolat = new LayOutMaker();
		calcolat.compute(3,maxSizeAgent, vett);

		tra.setTranslation(vett);
		
		try {
			super.setObjTransform(nome, tra);
		} catch (CadObjectNameException e1) {
			System.out.println("Nome Agente non valido"+nome);
		}
	}

	/**
	 * Inserisce un TC nell'universo visualizzandolo come una cubo di colore iniziale blu
	 * @param nome Il nome da associare al TC
	*/		
	public void addTC(String nome){
		
		Appearance app=new Appearance();
		//Assegna un Materiale all'aspetto
		Color3f color=new Color3f(1f,1f,1.0f);
		Color3f black=new Color3f(0f,0f,0f);
		app.setMaterial(new Material(color,black,color,black,80.0f));
		//Applico l'atialiasing
		LineAttributes lattr= new LineAttributes();
		lattr.setLineAntialiasingEnable(true);
		app.setLineAttributes(lattr);
		
		try {
			//Aggiungo un cubo all'univeso rappresentante il TupleCenter
			super.addUserObject(nome,new Cilindro(super.getSimUniverse().getCanvas(), 1.2f,4f, app, nome, elemSField));
			//lo aggiungo anche nella lista locale
			tcList.add(nome);

			//aggiorno la maxsize
			maxSizeTC=maxSizeTC+1;
			
		} catch (CadObjectNameException e) {
			System.out.println("Nome TC non valido:"+nome);		
		}
		//Lo pongo nella posizione pari a maxSize
		Transform3D tra=new Transform3D();
		Vector3d vett = new Vector3d(0,0,0);
		//Calcolo la posizione dovemetterlo
		LayOutMaker calcolat = new LayOutMaker();
		calcolat.compute(10,maxSizeTC, vett);

		tra.setTranslation(vett);
		try {
			super.setObjTransform(nome, tra);
		} catch (CadObjectNameException e1) {
			System.out.println("Nome Agente non valido"+nome);
		}
	}


	/**
	 * Elimina un Agente nell'universo 
	 * @param nome Il nome dell'Agente da eliminare
	*/		
	public void deleteAgent(String nome){
		try {
			super.deleteCadObject(nome);
			//elimino il nome anche dalla lista locale
			aList.remove(nome);

		} catch (CadObjectNameException e) {
			System.out.println("Nome Agente non valido"+nome);
		}
	}
	
	/**
	 * Elimina un TC nell'universo 
	 * @param nome Il nome del TC da eliminare
	*/		
	
	public void deleteTC(String nome){
		try {
			super.deleteCadObject(nome);
			//elimino il nome anche dalla lista locale
			tcList.remove(nome);

		} catch (CadObjectNameException e) {
			System.out.println("Nome TC non valido"+nome);
		}
	}

	/**
	 * Modifica il colore dell'Agente di nome specificato
	 * @param nome Il nome dell'Agente
	 * @param r La componente rossa del colore
	 * @param g La componente gialla del colore
	 * @param b La componente blu del colore
	*/		
	public void colorAgent(String nome, float r, float g, float b){
		Appearance app;
		try {
			//prima di settare il colore devo acquisire l'apparenza attuale dell'Agente
			//e poi impostare il colore lasciando cos inalterate eventuali settaggi all'apparenza 
			app = super.getObjAppearance(nome);
			//Assegna un Materiale all'aspetto
			Color3f color=new Color3f(r,g,b);
			Color3f black=new Color3f(0f,0f,0f);
			app.setMaterial(new Material(color,black,color,black,80.0f));
			super.setObjAppearance(nome, app);

		} catch (CadObjectNameException e) {
			System.out.println("Nome Agente non valido"+nome);
		}
	}
	/**
	 * Modifica il colore del TC di nome specificato
	 * @param nome Il nome del TC
	 * @param r La componente rossa del colore
	 * @param g La componente gialla del colore
	 * @param b La componente blu del colore
	*/		
	public void colorTC(String nome, float r, float g, float b){
		Appearance app;
		try {
			//prima di settare il colore devo acquisire l'apparenza attuale dell'Agente
			//e poi impostare il colore lasciando cos inalterate eventuali settaggi all'apparenza 
			app = super.getObjAppearance(nome);
			//Assegna un Materiale all'aspetto
			Color3f color=new Color3f(r,g,b);
			Color3f black=new Color3f(0f,0f,0f);
			app.setMaterial(new Material(color,black,color,black,80.0f));
			super.setObjAppearance(nome, app);
		} catch (CadObjectNameException e) {
			System.out.println("Nome TC non valido"+nome);
		}
	}
	
	/**
	 * Modifica la tasparenza dell'Agente di nome specificato
	 * @param nome Il nome dell'Agente
	 * @param t Indice di trasparenza(range : 0.0-1.0)
	*/		
	public void trasparenceAgent(String nome, float t){
		Appearance app;
		try {
			//prima di settare la Trasparenza devo acquisire l'apparenza attuale dell'Agente
			//e poi impostarla lasciando cos inalterate eventuali settaggi all'apparenza 
			app = super.getObjAppearance(nome);
			app.setTransparencyAttributes(new TransparencyAttributes(TransparencyAttributes.NICEST,t));
			super.setObjAppearance(nome, app);
		} catch (CadObjectNameException e) {
			System.out.println("Nome Agente non valido"+nome);
		}
	}
	/**
	 * Modifica la tasparenza del TC di nome specificato
	 * @param nome Il nome del TC
	 * @param t Indice di trasparenza(range : 0.0-1.0)
	*/		
	public void trasparenceTC(String nome, float t){
		Appearance app;
		try {
			//prima di settare la Trasparenza devo acquisire l'apparenza attuale dell'Agente
			//e poi impostarla lasciando cos inalterate eventuali settaggi all'apparenza 
			app = super.getObjAppearance(nome);
			app.setTransparencyAttributes(new TransparencyAttributes(TransparencyAttributes.NICEST,t));
			super.setObjAppearance(nome, app);
		} catch (CadObjectNameException e) {
			System.out.println("Nome TC non valido"+nome);
		}
	}

	
	/**
	 * Il metodo elimina tutti gli agenti e i tuplecenter visualizzati e va a ridistribuirli attraverso il layoutmaker
	 *  
	 * @param aglist Lista degli agenti presenti
	 * @param tupglist Lista dei tuple Center presenti
	*/		
	public void initialize(List agList, List tupList){
		//Inizializzo le liste locali
		for (int i=0; i< agList.size();i++) aList.add(agList.get(i).toString());	
		for (int b=0; b< tupList.size();b++) tcList.add(tupList.get(b).toString());

		//Elimino tutti gli elementi presenti nella scena
		super.deleteAllObject();
		
		//Azzero la maxSizeTC e porto a 24 la maxSizeAgent per far disporre gli agenti apartire dal quarto livello 
		//verranno aumentate ad ogni chiamata dei metodi 'add..'
		maxSizeAgent=24;
		maxSizeTC=0;
		String tcName;
		String agentName;
		
		Transform3D tran=new Transform3D();
		LayOutMaker calcolatore = new LayOutMaker();
		Vector3d vettore = new Vector3d(0,0,0);
		
		//Inserisce tanti Agenti quanti sono i nomi nella lista passata come parametro del costruttore
		for(int c =0; c<agList.size(); c++){
			//Aggiungo un agente
			agentName = agList.get(c).toString();
			addAgent(agentName);
			
		//FORSE STA PARTE  INUTILE INFATTI LA POSIZIONE DELL'AGENTE LA DECIDE ANCHE IL metodo addAgent...VERIFICARE 
			//Lo pongo nellaposizione decisa dal layoutMaker partendo da 24+1(quarto livello)
			//e con una distanza minima di 3
			calcolatore.compute(3,c+25, vettore);
			tran.setTranslation(vettore);
			try {
				super.setObjTransform(agentName, tran);
			} catch (CadObjectNameException e1) {
				System.out.println("Nome Agente non valido"+agentName);
			}
		}
		
		//Inserisce tanti TupleCenter quanti sono i nomi nella lista passata come parametro del costruttore
		for(int i=0; i<tupList.size(); i++){
			//Aggiungo un TC
			tcName = tupList.get(i).toString();
			addTC(tcName);
			
		//FORSE STA PARTE  INUTILE INFATTI LA POSIZIONE DELL'AGENTE LA DECIDE ANCHE IL metodo addTC...VERIFICARE 
			//Lo pongo nellaposizione decisa dal layoutMaker partendo da 0 (i=0) ma con una distanza di 10
			//per distanziare abbastanza un TC da un altro
			calcolatore.compute(10,i+1, vettore);
			tran.setTranslation(vettore);
			try {
				super.setObjTransform(tcName, tran);
			} catch (CadObjectNameException e1) {
				System.out.println("Nome tc non valido"+tcName);
			}
		}
		
	}
}
