package model;

/**
 *
 * La classe astratta permette di definire un generico oggetto adatto ad essere utilizzato nel CAD.
 * La classe crea la struttura generica e comune a tutti gli oggetti e lascia poi astratti alcuni metodi
 * che permettono di accedere a funzioni specifiche delle diverse geometrie.
 * Il metodo <code>getRoot</code> permette di ottenere il nodo padre di tutta la struttura, il 
 * metodo <code>setPickability</code>) permette di rendere o meno l'oggetto ruotabile e trslabile, 
 * il metodo  <code>setTrasformation</code> permette di applicare una trasformazione all'oggetto.
 * Il metodo  <code>getPosition</code> ritorna la posizione spaziale del centro dell'oggetto.
 * Infine dichiara il metodo atratto <code>setApparence</code> che permette di impostare le propriet 
 * visive dell'oggetto.
 * L'implementazione varier al variare della tecnica usata per creare il 
 * visual object(primitive o ridefinizione di Shape3D)
 * 
 * @author Matteo Campana nmatr.0000197228
 */

import javax.media.j3d.*;
import com.sun.j3d.utils.picking.behaviors.*;
import com.sun.j3d.utils.picking.PickTool;
import view.Listener.*;
import javax.swing.*;
import javax.vecmath.*;

public abstract class CadObject{
	//Dichiariamo gli elementi comuni a tutti i CadObject cio: un BG, un TG, e i behavior per il picking
	//i primi tre oggetti li ho resi publici in modo da potergli attaccare qualcosa anche in seguito
	//chiaramente prima di aver fatto il detach del bg.
	public BranchGroup bg;
	public TransformGroup tg;
	public  TransformGroup tg2;//Rispetto alla versione originale inserisco un altro TG per trasformazioni varie(es.rotazione)
								//tg2 sar figlio di tg
	protected PickRotateBehavior rotate;
	protected PickTranslateBehavior traslate;
	protected PickZoomBehavior zoom;
	protected Canvas3D canvas;
	protected String name;
	protected JTextField objectSField;
	/**
	 *Costruttore che crea la struttura comune dei vari CadObjects 
	 *Riceve la il canvas dell'universo chiamante che servir per il piking con mouse
	 *Riceve la il JTextField sul quale scrivere il nome dell'oggetto selezionato
	 *Riceve il nome da visualizzare quando sar selezionato l'oggetto
	 *Gli ultimi due parametri serviranno al listener di callback del picking.
	 * @param can Riceve il canvas dell'universo chiamante
	 * @param objectSelField Riceve il JTextField sul quale scrivere il nome dell'oggetto selezionato 
	 * @param selectingName Riceve il nome da visualizzare quando sar selezionato l'oggetto
	 */

	public CadObject(Canvas3D can,String selectingName, JTextField objectSelField){
		
		//Acquisisco il Canvas3D dell'universo chiamante;
		canvas = can;
		//Imposto il nome del cad object come il nome di selezione desiderato
		name=selectingName;
		//Acquisisco il JTextField dove scrivere l'oggetto selezionato con il mouse
		objectSField= objectSelField;
		//Creo il TG
		tg = new TransformGroup();
		tg2 = new TransformGroup();
		
		//Imposto il TG in modo che permetta le trasformazioni
		tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		tg.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		tg.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
		tg.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		//Questa serve per ottenere il bound e quindi la posizione "Locale" del tg
		tg.setCapability(TransformGroup.ALLOW_BOUNDS_READ);

		//Questa capability serve per fare includere il gruppo nel path del picking
		tg.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
		
		//Imposto il TG2 in modo che permetta le trasformazioni
		tg2.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		tg2.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		tg2.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
		tg2.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
		//Questa capability serve per fare includere il gruppo nel path del picking
		tg2.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
		//Questa serve per ottenere il bound e quindi la posizione "Locale" del tg
		tg2.setCapability(TransformGroup.ALLOW_BOUNDS_READ);

		
		//Creo il BG
		bg = new BranchGroup();
		//permette al BG di essere scollegato
		bg.setCapability(BranchGroup.ALLOW_DETACH);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		bg.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		bg.setCapability(BranchGroup.ALLOW_AUTO_COMPUTE_BOUNDS_READ);
		//Questa serve per ottenere il bound e quindi la posizione del bg che  poi 
		//la posizione "Globale" dell'oggetto poich ilbg  la root dlla struttura di un cad object
		bg.setCapability(BranchGroup.ALLOW_BOUNDS_READ);

		//creo i pick behavior in modo che agiscano sul BG
		rotate = new PickRotateBehavior(bg,canvas,new BoundingSphere(new Point3d(0.0,0.0,0.0),1000.0));
		traslate = new PickTranslateBehavior(bg,canvas,new BoundingSphere(new Point3d(0.0,0.0,0.0),1000.0));
		zoom = new PickZoomBehavior(bg,canvas,new BoundingSphere(new Point3d(0.0,0.0,0.0),1000.0));
		
		//FACCIO ORA IN MODO CHE SIA POSSIBILE SELEZIONARE UN OGGETTO CON IL MOUSE
		//Crea un "listener" di callback del picking che associa il nome selectingName all'oggetto "pickato"
		SelectionListener select= new SelectionListener(name, tg2, objectSField);
		//Registra il listener ai 3 behavior del picking(Zoom, traslazione, rotazione)
		rotate.setupCallback(select);
		traslate.setupCallback(select);
		zoom.setupCallback(select);

		
		//Imposta la modalit di picking tramite Bounds
		//Rispetto all'originale ho utilizzato la modalit Bound per evitare il problema che si presentava
		//quando si inserivano oggetti con dimensioni identiche.
		//Il problema era infatti dovuto all'uso del metodo Geometry del picking.
		rotate.setMode(PickTool.BOUNDS);
		traslate.setMode(PickTool.BOUNDS);
		zoom.setMode(PickTool.BOUNDS);
		

		
		//Assemblo la struttura
		tg.addChild(tg2);
		bg.addChild(tg);
		bg.addChild(rotate);
		bg.addChild(traslate);
		bg.addChild(zoom);
		

	}
	
	/**
	 * 
	 *@return bg Ritorna il nodo padre del CadObject
	 * 
	 */	
	public Node getRoot(){
		return bg;
	}
	
	/**
	 * Disabilita o meno la capacit di essere PICKED.
	 * Il settaggio di questa capacit deve essere fatta prima che l'oggetto venga reso vivo.
	 * 
	 *@param set Se true permette all'oggetto di essere spostato attraverso picking del mouse.False nel caso opposto
	 * 
	 */	
	public void setPickability(boolean set){
		try{
			if (set){
				tg.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
				tg2.setCapability(TransformGroup.ENABLE_PICK_REPORTING);
			}else{
				tg.clearCapability(TransformGroup.ENABLE_PICK_REPORTING);
				tg2.clearCapability(TransformGroup.ENABLE_PICK_REPORTING);
			}
		}catch(RestrictedAccessException ex){
			System.out.println("Eccezione "+ex+" catturata");
			System.out.println("Tentativo di modificare la capability di un oggetto LIVE");
		}

   }
	
	/**
	 *@param transf Riceve la trasformazione da effettuare sull'oggetto
	 * 
	 */	
	public void setTransformation(Transform3D transf){
		try{
			Transform3D transformation = transf;
			tg.setTransform(transformation);
		}catch (CapabilityNotSetException ex){
			System.out.println("Eccezione "+ex+" catturata");
			System.out.println("Capability non attivata");
		}catch (BadTransformException ex){
			System.out.println("Eccezione "+ex+" catturata");
			System.out.println("Trasformazione non valida");
		}
	}
	
	/**
	 * Restituisce lo stato della trasformazione affine dell'oggetto
	 * 
	 *@return tra Ritorna la trasformazione attuale dell'oggetto
	 * 
	 */	
	public Transform3D getTransformation(){
		Transform3D tra = new Transform3D();
		tg.getTransform(tra);
		return tra;
	}
	
	/**
	 *Metodo per ottenere la posizione del CAdObject
	 *In particolare restituisce la posizione del centro dl bound associato alla Root(bg) dell'oggetto
	 * 
	 * @return point  Ritorna le coordinate del centro
	*/	
	public Point3d getPosition(){
		BoundingSphere hh=new BoundingSphere(bg.getBounds());
		Point3d point = new Point3d();
		hh.getCenter(point);
		return point;
	}

	
	/**
	 *Metodo astratto per impostare gli aspetti visivi dell'oggetto3D sia che esso sia una primitiva, sia
	 *uno shape3D definito.
	 *L'impostazione dell'apparenza nei due casi avr implementazionediversa
	 * 
	 */	
	public abstract void setAppearance(Appearance app);

	/**
	 *Metodo astratto per ottenere gli aspetti visivi dell'oggetto3D(Appearance dell'oggetto) 
	 *sia che esso sia una primitiva, sia
	 *uno shape3D ridefinito.
	 * 
	*/	
	public abstract Appearance getAppearance();

}
