package model;

/**
 * La classe implementa una serie di metodi e costruttori che permettono di gestire le entit presenti nel CAD.
 * Il metodo <code>getSimUniverse</code> permette di ottenere il simple universe con il quale si  creato 
 * la base per l'ambiente CAD; il metodo <code>addUserObject</code> permette di aggiungere un oggetto nell'universo;
 * il metodo <code>setObjTransform</code> permette di applicare una trasformazione affine all'oggetto; 
 * il metodo <code>setObjAppearance</code> permette di impostare all'oggetto alcune propriet visive; il metodo
 * <code>getObjAppearance</code> permette di acquisire le propriet visive dell'oggetto;
 * Il metodo <code>getObjTransform</code> acquisisce le trasformazioni affini appplicate all'oggetto;
 * Il metodo <code>getObjList</code> permette di ottenere la lista dei nomi di tutti gli oggetti utente
 * Il metodo <code>deleteCadObject</code> cancella un determinato oggetto.
 * Il metodo <code>deleteAllObject</code> cancella tuuti gli oggetti;il metodo <code>getObjList</code> permette di ottenere
 * il set dei nomi di tutti gli oggetti inseriti dall'utente
 * 
 * @author Matteo Campana nmatr.0000197228
 */

import java.util.*;
import java.awt.*;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.keyboard.*;
import com.sun.j3d.utils.scenegraph.io.*;
import javax.swing.*;


public class CadUniverse {

	public HashMap userobject;
	private SimpleUniverse simpleU;
	public BranchGroup scena;
	public BranchGroup BGambiente;	
	public BranchGroup BGuserobject;
	
	/**
	 * Il costruttore realizza una struttura costituita da un BG scena con due figli (BGambiente e BGuserobject)
	 * Poi realizza in automatico una serie di settaggi utili.
	 * Infine carica attraverso il metodo privato addAmbentObject una serie di CadObject che saranno presenti sin 
	 * dall'apertura dell'ambiente
	 * 
	*/	
	public CadUniverse(){
		
		//Crea i contenitori per gli oggetti utente 
		userobject = new HashMap();
		
		//Crea il BG scena
		scena = new BranchGroup();
		scena.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		scena.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		scena.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		scena.setCapability(BranchGroup.ALLOW_DETACH);
		
		//Crea il BG ambiente BGambiente
		BGambiente = new BranchGroup();
		BGambiente.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		BGambiente.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		BGambiente.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
		BGambiente.setCapability(BranchGroup.ALLOW_DETACH);
		
		//Crea il BG per gli oggetti utente BGuserobject
		BGuserobject = new BranchGroup();
		for(int i=0;i<17;i++){
			BGuserobject.setCapability(i);
		}
		//Collego BGuserobject e BGambiente come figli di scena
		scena.addChild(BGambiente);
		scena.addChild(BGuserobject);
		
		//Recupera le configurazioni grafiche del computer
		GraphicsConfiguration config=SimpleUniverse.getPreferredConfiguration();

		//Crea il Canvas3D
		Canvas3D canvas3D=new Canvas3D(config);
		
		//Crea il SimpleUniverse
		simpleU=new SimpleUniverse(canvas3D);

		//Recupero la TransformGroup della ViewingPlatform
		TransformGroup vpTG=simpleU.getViewingPlatform().getViewPlatformTransform();
//		simpleU.getViewingPlatform().setNominalViewingTransform();

		//Creo un behavior per la navigazione da tastiera
		KeyNavigatorBehavior keyNavBeh=new KeyNavigatorBehavior(vpTG);
		//Imposto il bound del behavior
		keyNavBeh.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
		//Aggiungo il behavior alla scena
		scena.addChild(keyNavBeh);
		
		//AGGIUNGO UNA SERIE DI OGGETTI DI CORREDO ALL'AMBIENTE
		//Aggiungo una griglia da usare come riferimento del piano dandogli un nome fittizio
		CadObject land = new Griglia(canvas3D, "griglia", new JTextField());
		//Applico l'atialiasing alla griglia
		Appearance appear=new Appearance();
		LineAttributes lattr= new LineAttributes();
		lattr.setLineAntialiasingEnable(true);
		appear.setLineAttributes(lattr);
		land.setAppearance(appear);
		//La rendo non PICKABLE
		land.setPickability(false);
		this.addAmbientObject(land);
		
		//Crea una luce direzionale
		BoundingSphere bounds2=new BoundingSphere(new Point3d(0.0,0.0,0.0),100);
		DirectionalLight lightD2=new DirectionalLight();
		lightD2.setInfluencingBounds(bounds2);
		lightD2.setColor(new Color3f(1,1,1));
		lightD2.setDirection(new Vector3f(0.0f,5.0f,5.5f));
		scena.addChild(lightD2);
		
		//Aggiunge la scena all'universo (rendo cio vivi gli oggetti)
		simpleU.addBranchGraph(scena);	
	}
	
	/**
	 * Restituisce il simpleuniverse
	 * @param simpleU Il simpleuniverse contenuto dal cad
	*/		
	public SimpleUniverse getSimUniverse(){
		return simpleU;
	}
	
	/**
	 * Usato dal costruttore per inserire degli oggetti nell'ambiente iniziale dell'universo
	 * @param obj Riceve il CadObject da inserire nell'ambiente del cad
	*/			
	private void addAmbientObject(CadObject obj){
		CadObject cadObject = obj;
		BGambiente.addChild(cadObject.getRoot());
	}
	
	/**
	 * Inserisce un CadObject nell'universo associandogli un nome.
	 * @param nome Il nome da associare all'oggetto
	 * @param obj L'oggetto da inserire
	 * @exception CadObjectNameException Eccezione lanciata se si inserisce un oggetto con nome uguale ad uno gi esistente
	*/		
	public void addUserObject(String nome, CadObject obj) throws CadObjectNameException{
		if (userobject.containsKey(nome)){ 
			throw new CadObjectNameException();
		}else{
			CadObject cadObject = obj;
			//inserisco l'oggetto nella collezione degli oggetti utente associandolo al nome passato
			userobject.put(nome, cadObject);
			//collego la radice dell'oggetto con il BG degli userobject
			BGuserobject.addChild(cadObject.getRoot());


		}
	}
	
	/**
	 * Setta la trasformazione affine all'oggetto associato al nome passato.
	 * @param nome Il nome dell'oggetto
	 * @param transform La trasformazione affine
	 * @exception CadObjectNameException Eccezione lanciata se non esiste un oggetto con il nome specificato
	*/		
	public void setObjTransform(String nome, Transform3D transform) throws CadObjectNameException{
		if (userobject.containsKey(nome)){ 
			//Estraggo dalla collezione degli oggetti utente l'oggetto associato al nome passato
			CadObject cadObject = (CadObject)userobject.get(nome);
			//setto la trasformazione affine dell'oggetto
			cadObject.setTransformation(transform);
		}else{
			throw new CadObjectNameException();
		}
	}
	
	/**
	 * Setta gli aspetti visivi dell'oggetto che ha il nome passato.
	 * @param nome Il nome dell'oggetto
	 * @param ap La Apparenza dell'oggetto
	 * @exception CadObjectNameException Eccezione lanciata se non esiste un oggetto con il nome specificato
	*/		
	public void setObjAppearance(String nome, Appearance ap) throws CadObjectNameException{
		if (userobject.containsKey(nome)){ 
			//Estraggo dalla collezione degli oggetti utente l'oggetto associato al nome passato
			CadObject cadObject = (CadObject)userobject.get(nome);
			//setto l'Appearance dell'oggetto
			cadObject.setAppearance(ap);
		}else{
			throw new CadObjectNameException();
		}
	}
	
	/**
	 * Ritorna L'appearance dell'oggetto che ha il nome passato.
	 * @param nome Il nome dell'oggetto
	 * @return L' Apparenza dell'oggetto
	 * @exception CadObjectNameException Eccezione lanciata se non esiste un oggetto con il nome specificato
	*/		
	public Appearance getObjAppearance(String nome) throws CadObjectNameException{
		if (userobject.containsKey(nome)){ 
			//Estraggo dalla collezione degli oggetti utente l'oggetto associato al nome passato
			CadObject cadObject = (CadObject)userobject.get(nome);
			//ritorno l'apparenza dell'oggetto
			return cadObject.getAppearance();
		}else{
			throw new CadObjectNameException();
		}
	}

	/**
	 * Ritorna la trasformazione affine dell'oggetto che ha il nome passato.
	 * @param nome Il nome dell'oggetto
	 * @return La trasformazione dell'oggetto
	 * @exception CadObjectNameException Eccezione lanciata se non esiste un oggetto con il nome specificato
	*/		
	public Transform3D getObjTransform(String nome) throws CadObjectNameException{
		if (userobject.containsKey(nome)){ 
			//Estraggo dalla collezione degli oggetti utente l'oggetto associato al nome passato
			CadObject cadObject = (CadObject)userobject.get(nome);
			//ritorno la trasformazione dell'oggetto
			return cadObject.getTransformation();
		}else{
			throw new CadObjectNameException();
		}
	}

	/**
	 * Elimina l'oggetto.
	 * @param nome Il nome dell'oggetto
	 * @exception CadObjectNameException Eccezione lanciata se non esiste un oggetto con il nome specificato
	*/		
	public void deleteCadObject(String nome) throws CadObjectNameException{
		if (userobject.containsKey(nome)){ 
			//Estraggo dalla collezione degli oggetti utente l'oggetto associato al nome passato
			CadObject cadObject = (CadObject)userobject.get(nome);
			//Scollego la radice dell'oggetto dall'universo
			BranchGroup BG = (BranchGroup) cadObject.getRoot();
			BG.detach();
			//Elimino l'oggetto dalla collezione userObject
			userobject.remove(nome);
		}else{
			throw new CadObjectNameException();
		}
	}
	
	/**
	 * Elimina tutti gli oggetti.
	 *
	*/		
	public void deleteAllObject(){
		Iterator it=userobject.values().iterator();
		CadObject obj;
		while(it.hasNext()){
			obj=(CadObject)it.next();
			BranchGroup BG = (BranchGroup)obj.getRoot();
			BG.detach();
			it.remove();
		}
	}
	
	/**
	 * Permette di ottenere la lista dei nomi di tutti gli oggetti
	 * @return Il set di nomi
	 *
	*/		
	public Set getObjList(){
		return userobject.keySet();
	}
	
	/**
	 * Permette di salvare la scena corrente
	 *
	*/		
	public void saveScene(String nomeFile){
		try{
			//Salvo l'universo
			SceneGraphFileWriter save = new SceneGraphFileWriter(new java.io.File(nomeFile),this.getSimUniverse(),true,"CAD by Teo", "Cad by Teo");
			//Assegno un nome al nodo degli oggetti utente per poterlo poi ritrovara
			save.addObjectName("BGuserobject",BGuserobject);
			save.close();
		}catch (java.io.IOException e){
			System.out.println("Eccezione "+e+" catturata");
			System.out.println("Errore di scrittura file");
		}catch (UnsupportedUniverseException c){
			System.out.println("Eccezione "+c+" catturata");
			System.out.println("Universo non supportato per il salvataggio");
		}catch (NamedObjectException f){
			System.out.println("Eccezione "+f+" catturata");
			System.out.println("Nome oggetto scenegraph non corretto");
		}	
	}

	/**
	 * Permette di caricare da file la scena 
	 *
	*/		
	public void loadScene(String nomeFile){
		try{
			//Carico da file la scena salvata
			SceneGraphFileReader load = new SceneGraphFileReader(new java.io.File(nomeFile));
			BranchGroup[] BGListTemp = load.readAllBranchGraphs();
			//Riottengo gli userobject salvati
			BranchGroup BGloaded = (BranchGroup)load.getNamedObject("BGuserobject");
			//Li scollego dal padre originale
			BGloaded.detach();
			//lo aggiungo agli oggetti utente
			BGuserobject.addChild(BGloaded);
			load.close();
		}catch (java.io.IOException e){
			System.out.println("Eccezione "+e+" catturata");
			System.out.println("Errore di lettura file");
		}catch (ObjectNotLoadedException c){
			System.out.println("Eccezione "+c+" catturata");
			System.out.println("Universo non adatto alla lettura");
		}catch (NamedObjectException f){
			System.out.println("Eccezione "+f+" catturata");
			System.out.println("Nome oggetto scenegraph non corretto");
		}	
	}

}
