package model;

/**
 *
 * La classe concreta permette di definire un generico cubo adatto ad essere utilizzato nel CAD.
 * La classe crea implementa due metodi: <code>setAppearance</code> che permette di impostare 
 * le propriet visive dell'oggetto e <code>getAppearance</code> che ritorna l'apparenza dell'oggetto.
 * 
 * @author Matteo User
 */

import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.Box;
import javax.swing.*;
public class Cubo extends CadObject{
	private Box cubo;
	
	/**
	 *Costruttore che crea la struttura comune dei vari CadObjects attraverso il super e poi crea 
	 *un parallelepipedo di dimensioni specificate e con una certa apparenza
	 * 
	 */
	public Cubo(Canvas3D canvas, float xdim, float ydim, float zdim, Appearance ap, String name,JTextField objectSelField){
		super(canvas, name, objectSelField);
		
		//Attivo tutte le capability dell'appearance.
		for(int i=0;i<22;i++){
			ap.setCapability(i);
		}
	
		//crea il cubo con certe dimensioni e una certa apparenza
		cubo = new Box(xdim, ydim, zdim, Box.GENERATE_NORMALS, ap);
		
		//Imposta la capability di modificare l'aspetto
		cubo.setCapability(Box.ENABLE_APPEARANCE_MODIFY);
		//Il cubo ha 6 shape diversi(uno per ogni faccia)
		for(int i=0; i<6; i++){
			//Imposta la capacit dell'aspetto del cubo
			cubo.getShape(i).setAppearanceOverrideEnable(true);
			cubo.getShape(i).setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
			cubo.getShape(i).setCapability(Shape3D.ALLOW_APPEARANCE_READ);
			//Per poter utilizzare le Geometry per il picking, si deve impostare la capability
			//per la generazione delle intersezioni tra oggetti
			//Enumeration enum=(cubo.getShape(i)).getAllGeometries();
			//while (enum.hasMoreElements()) ((Geometry)enum.nextElement()).setCapability(Geometry.ALLOW_INTERSECT);
		}
		
		//aggiungo la primitiva come figlio di TG.
		tg2.addChild(cubo);
	}
	
	/**
	 *Metodo astratto per impostare gli aspetti visivi dell'oggetto3D
	 * 
	 * 
	 * @param app Riceve l'apparenza da impostare nell'oggetto
	 */	
	public void setAppearance(Appearance app){
		cubo.setAppearance(app);
	}
	
	/**
	 *Metodo astratto per ottenere gli aspetti visivi dell'oggetto3D(Appearance dell'oggetto) 
	 *sia che esso sia una primitiva, sia
	 *uno shape3D ridefinito.
	 * 
	 * @return  Ritorna L'apparenza
	*/	
	public Appearance getAppearance(){
		return cubo.getAppearance();
	}
}
