package model;

/**
 * La classe concreta permette di definire una griglia adatta ad essere utilizzato nel CAD.
 * La classe implementa due metodi: <code>setAppearance</code> che permette di impostare 
 * le propriet visive dell'oggetto e <code>getAppearance</code> che ritorna l'apparenza dell'oggetto.
 * 
 * @author Matteo User
 * 
 */

import javax.media.j3d.*;
import model.visualobj.*;
import javax.swing.*;

public class Griglia extends CadObject{
	private Shape3D griglia;
	
	/**
	 *Costruttore che crea la struttura comune dei vari CadObjects attraverso il super e poi crea 
	 *una griglia specificate e con una certa apparenza
	 * 
	 */
	public Griglia(Canvas3D canvas, String name,JTextField objectSelField){
		super(canvas, name, objectSelField);
		
		//crea la griglia con una specifica apparenza
		griglia = new Grid();
		
		//Attivo tutte le capability dell'appearance.
		for(int i=0;i<22;i++){
			griglia.getAppearance().setCapability(i);
		}
		
		//Imposta la capability di modificare l'aspetto
		griglia.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		griglia.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
		

		//Per poter utilizzare le Geometry per il picking, si deve impostare la capability
		//per la generazione delle intersezioni tra oggetti
		//griglia.setCapability(Geometry.ALLOW_INTERSECT);

		
		//aggiungo la primitiva come figlio di TG.
		tg2.addChild(griglia);
	}
	
	/**
	 *Metodo astratto per impostare gli aspetti visivi dell'oggetto3D
	 * 
	 * 
	 * @param app Riceve l'apparenza da impostare nell'oggetto
	 */	
	public void setAppearance(Appearance app){
		griglia.setAppearance(app);
	}
	
	/**
	 *Metodo astratto per ottenere gli aspetti visivi dell'oggetto3D(Appearance dell'oggetto) 
	 *sia che esso sia una primitiva, sia
	 *uno shape3D ridefinito.
	 * 
	 * @return  Ritorna L'apparenza
	*/	
	public Appearance getAppearance(){
		return griglia.getAppearance();
	}

}

