package model;

/**
 * Estende la classe <code>CadUniverse</code> rendendola adatta a rappresentare l'insieme dei 
 * nodi presenti nel sistema distribuito da monitorare.
 * Il metodo  <code>addHost</code>  utilizzato per inserire nell'ambiente grafico un nodo rappresentato da
 * una sfera di color rosso.
 * Il metodo  <code>deleteHost</code>  utilizzato per eliminare nell'ambiente grafico un nodospecifico
 * Il metodo  <code>colorHost</code>  utilizzato per cambiare colore ad uno specifico nodo.
 * Il costruttore inserisce tanti Host quanti sono i nomi nella lista passata come parametro del costruttore
 * 
 * @author Matteo Campana nmatr.0000197228
 */
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import javax.swing.*;

public class HostUniverse extends CadUniverse {
	//max size indica il max numero di nodi presenti nell'ambiente
	//serve per posizionare spazialmente un eventuale nuovo nodo in una posizione nella quale si 
	//sicuri non sia mai stato inserito un nodo...L'approccio  un po laborioso ma evita spiacevoli sovraposizioni 
	//Viene 'rinfrescata' se si lancia il metodo initialize
	protected int maxSize;
	protected JTextField nodeSField;
	/**
	 * il costruttore inserisce gli host presenti attualmente nell'ambiente grafico
	 * distribuendoli concentricamente.
	 * Riceve il JTextField sul quale verr visualizzato il nome dell'oggetto selezionato con il mouse
	 * 
	 * @param HostList La lista dei nodi presenti attualmente
	 * @param nodeSelField Riceve il JTextField sul quale verr visualizzato il nome dell'oggetto selezionato con il mouse
	*/		
	public  HostUniverse(List hostList, JTextField nodeSelField){
		super();
		nodeSField=nodeSelField;
		this.initialize(hostList);
	}
	
	/**
	 * Inserisce un Host nell'universo visualizzandolo come una sfera di colore rosso
	 * 
	 * @param nome Il nome da associare all'Host
	*/		
	public void addHost(String nome){
		Appearance app=new Appearance();
		
		//Assegna un Materiale di color rosso all'aspetto
		Color3f color=new Color3f(1.0f,0f,0f);
		Color3f black=new Color3f(0f,0f,0f);
		app.setMaterial(new Material(color,black,color,black,80.0f));
		//lo rendo wired
		PolygonAttributes pa=new PolygonAttributes();
		pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
		app.setPolygonAttributes(pa);
		//Applico l'atialiasing e allargo la dimensione delle linee
		LineAttributes lattr= new LineAttributes(1.4f, LineAttributes.PATTERN_SOLID, true);
		app.setLineAttributes(lattr);

		try {
			//Lo aggiungo all'universo
			super.addUserObject(nome,new Sfera(super.getSimUniverse().getCanvas(), 2f, app, nome, nodeSField));
			//aggiorno la maxsize
			maxSize=maxSize+1;
			//Lo pongo nella posizione pari a maxSize
				Transform3D tra=new Transform3D();
				Vector3d vett = new Vector3d(0,0,0);
				//Calcolo la posizione spaziale nella quale inserirlo
				LayOutMaker calcolat = new LayOutMaker();
				calcolat.compute(10,maxSize, vett);
				tra.setTranslation(vett);
				try {
					super.setObjTransform(nome, tra);
				} catch (CadObjectNameException e1) {
					System.out.println("Nome nodo non valido");
				}
			//Lo faccio Ruotare continuamente
				//ottengo il CadObject associato al nodo creato
				CadObject nod=(CadObject)super.userobject.get(nome);
				//Lo distacco il root del cad object dalla scena
				nod.bg.detach();
				
				//Crea un timer
				Alpha rotationAlpha=new Alpha(-1,3500);
				//Crea un interpolatore per le rotazioni collegato con il gruppo di trasformazione
				RotationInterpolator rotator=new RotationInterpolator(rotationAlpha,nod.tg2);
				//Imposta un raggio d'azione all'interpolatore
				BoundingSphere bounds=new BoundingSphere(new Point3d(0.0,0.0,0.0),1000.0);
				rotator.setSchedulingBounds(bounds);
				//aggiunge l'interpolatore alla gruppo di trasformazione
				nod.tg2.addChild(rotator);
			
				//riattacco il BG root del cad object con la scena dell'universo
				super.BGuserobject.addChild(nod.getRoot());
			
		} catch (CadObjectNameException e) {
			System.out.println("Nome nodo non valido");		
		}
	
	}

	/**
	 * Elimina un Host dell'universo 
	 * @param nome Il nome dell'Host da eliminare
	*/		
	public void deleteHost(String nome){
		try {
			super.deleteCadObject(nome);
		} catch (CadObjectNameException e) {
			System.out.println("Nome nodo non valido");
		}
	}
	
	/**
	 * Modifica il colore del nodo con nome specificato
	 * @param nome Il nome dell'Host
	 * @param r La componente rossa del colore
	 * @param g La componente gialla del colore
	 * @param b La componente blu del colore
	*/		
	public void colorHost(String nome, float r, float g, float b){
		Appearance app;
		try {
			//prima di settare il colore devo acquisire l'apparenza attuale del nodo
			//e poi impostare il colore lasciando cos inalterate eventuali settaggi all'apparenza 
			app = super.getObjAppearance(nome);
			//Assegna un Materiale all'aspetto
			Color3f color=new Color3f(r,g,b);
			Color3f black=new Color3f(0f,0f,0f);
			app.setMaterial(new Material(color,black,color,black,80.0f));
			super.setObjAppearance(nome, app);
		} catch (CadObjectNameException e) {
			System.out.println("Nome nodo non valido");
		}
	}

	
	/**
	 * Il metodo elimina tutti i nodi presenti e distribuisce spazialmente i nodi (attraverso il layoutmaker) 
	 * specificati nella lista hList.
	 * Il metodo  utile per ridistribuire in modo ordinato i nodi presenti 
	 * Il metodo viene invocato anche dal costruttore  
	 * 
	 * @param hlist degli host presenti
	*/		
	public void initialize(List hList){
		//Elimino tutti i nodi presenti
		super.deleteAllObject();
		//Azzero la maxSize che verr aumentata ad ogni chiamata del metodo addHost
		maxSize=0;
		
		String hostName;
		Transform3D tran=new Transform3D();
		LayOutMaker calcolatore = new LayOutMaker();
		Vector3d vettore = new Vector3d(0,0,0);
		
		//Inserisce tanti Host quanti sono i nomi nella lista passata come parametro
		for(int i=0; i<hList.size(); i++){
			//Aggiungo un nodo
			hostName = hList.get(i).toString();
			addHost(hostName);
		//FORSE STA PARTE  INUTILE INFATTI LA POSIZIONE DELL'AGENTE LA DECIDE ANCHE IL metodo addHost...VERIFICARE 
			//Lo pongo nellaposizione decisa dal layoutMaker e ad una distanza minima di 10
			calcolatore.compute(10,i+1, vettore);
			tran.setTranslation(vettore);
			try {
				super.setObjTransform(hostName, tran);
			} catch (CadObjectNameException e1) {
				System.out.println("Nome nodo non valido");
			}
		}
	}
}
