package model;

/**
 *
 * La classe concreta permette di definire una generica sfera adatto ad essere utilizzato nel CAD.
 * La classe crea implementa due metodi: <code>setAppearance</code> che permette di impostare 
 * le propriet visive dell'oggetto e <code>getAppearance</code> che ritorna l'apparenza dell'oggetto.
 * 
 * @author Matteo User
 */

import javax.media.j3d.*;
import com.sun.j3d.utils.geometry.*;
import javax.swing.*;
public class Sfera extends CadObject{
	private Sphere sfera;
	
	/**
	 *Costruttore che crea la struttura comune dei vari CadObjects attraverso il super e poi crea 
	 *una sfera di raggio specificato e con una certa apparenza
	 *
	 */
	public Sfera(Canvas3D canvas, float ratio, Appearance ap, String name,JTextField objectSelField){
		super(canvas, name, objectSelField);
		
		//Attivo tutte le capability dell'appearance.
		for(int i=0;i<22;i++){
			ap.setCapability(i);
		}
		
		//crea la sfera con certe dimensioni e una certa apparenza
		sfera = new Sphere(ratio, Sphere.GENERATE_NORMALS, 30, ap);
		
		//Imposta la capability di modificare l'aspetto
		sfera.setCapability(Sphere.ENABLE_APPEARANCE_MODIFY);
		
		//Imposta la capacit di sovrascrivere l'aspetto della sfera
		sfera.getShape().setAppearanceOverrideEnable(true);
		sfera.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		sfera.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_READ);
		
		
		//Per poter utilizzare le Geometry per il picking, bisogna impostare la capability
		//per la generazione delle intersezioni tra oggetti
		//Enumeration enum=(sfera.getShape()).getAllGeometries();
		//while (enum.hasMoreElements()) ((Geometry)enum.nextElement()).setCapability(Geometry.ALLOW_INTERSECT);
		
		//aggiungo la primitiva come figlio di TG.
		tg2.addChild(sfera);
		
	}
	
		
	/**
	 *Metodo astratto per impostare gli aspetti visivi dell'oggetto3D
	 * 
	 * 
	 * @param app Riceve l'apparenza da impostare nell'oggetto
	 */	
	public void setAppearance(Appearance app){
		sfera.setAppearance(app);
	}
	
	/**
	 *Metodo astratto per ottenere gli aspetti visivi dell'oggetto3D(Appearance dell'oggetto) 
	 *sia che esso sia una primitiva, sia
	 *uno shape3D ridefinito.
	 * 
	 * @return  Ritorna L'apparenza
	*/	
	public Appearance getAppearance(){
		return sfera.getAppearance();
	}
}
