package view;

/**
 * Questa classe permette di lanciare l'applicazione di monitoraggio dell'organizzazione (insieme di tutti i nodi)
 * e indirettamente (attraverso la selezione di un nodo e la pressione di un pulsante nel pannello) 
 * permette di lanciare anche l'applicazione di monitoraggio di uno specifico nodo.
 * La classe pu essere vista come l'agente che ha il compito di ottenere le informazioni relative all'organizzazione intera
 * interagendo con il tucson.
 * In particolare l'agente si aspetta di ricevere nel tuple center "monitoring" nell'host "locale" tutte 
 * le informazioni di cui ha bisogno.
 * Come gi accennato l'applicazione permette anche di creare anche gli agenti responsabili del monitoraggio di specifici nodi.
 * E' chiaro che questo agente dovr avere un flusso di controllo separato dagli agenti che creer in seguito.
 * Questo  reso possibile attraverso l'utilizzo di Thread.
 * 
 * @author Matteo Campana nmatr.0000197228
 *
 */
import model.*;
import javax.swing.*;
import java.awt.*;
import alice.tucson.api.*;
import alice.logictuple.*;
import alice.logictuple.Var;
import alice.tuprolog.*;

import java.io.*;
public class GraficHostAgent {
	public static void main(String[] args){

	try{
		//Lancio la teoria prolog
		try {
		FileInputStream f=new FileInputStream("theory.pl");
		Prolog Pengine=new Prolog();
		Pengine.setTheory(new Theory(f));
		
		SolveInfo sol=null;
		sol = Pengine.solve("go.");
		while (sol.isSuccess()){
			System.out.println("solution: "+sol.getSolution());
			if (Pengine.hasOpenAlternatives()){
				sol=Pengine.solveNext();
			} else{
				break;
			}
		}		 
		} catch (InvalidTheoryException e1) {
		System.out.println("Errore nella teoria" );
		e1.printStackTrace();
		}catch (MalformedGoalException e2) {
		System.out.println("Errore sintassi del Goal" );
		e2.printStackTrace();
		}catch (NoSolutionException e3) {
		System.out.println("Errore nella soluzione" );
		e3.printStackTrace();
		} catch (FileNotFoundException e4) {
			System.out.println("Errore apertura file" );
			e4.printStackTrace();
		} catch (IOException e5) {
			System.out.println("Errore file" );
			e5.printStackTrace();
		} catch (NoMoreSolutionException e6) {
			System.out.println("Errore nella ricerca di nextSolution" );
			e6.printStackTrace();
		}
		//Creo l'interfaccia grafica
		HostGui myGui= new HostGui();
		JFrame f = new JFrame("Organization monitoring");
		f.getContentPane().add(myGui, BorderLayout.EAST);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setExtendedState(Frame.MAXIMIZED_BOTH);
		
		//Creo l'agente "graficHosts"  nel nodo locale
		AgentId aid=new AgentId("graficHosts");
		TucsonContext cn = Tucson.enterContext(new DefaultContextDescription(aid));
		
		//Abilito l'osservabilit
		//TupleCentreId tenabling = new TupleCentreId("'$ORG'");
		//cn.out(tenabling, new LogicTuple("cmd",new Value("enable_observability")));



		TupleCentreId tid = new TupleCentreId("'$OBS'");
		
		//Leggo dal tuple center ''$OBS'' la tupla nodelist(X)
		//contenente gli indirizzi dei nodi presenti attualmente
		LogicTuple t1 =cn.rd(tid, new LogicTuple("node_list",new Var("X")));
		
		//Creo un Universo con i nodi presenti passandogli la lista dei nomi dei nodi
		HostUniverse myUniverse = new HostUniverse(t1.getArg(0).toList(), myGui.nodeSel);
		//Collego l'universo con l'interfaccia grafica
		f.getContentPane().add(myUniverse.getSimUniverse().getCanvas(), BorderLayout.CENTER);
		f.show();
		//Ora lo metto in ascolto di informazioni relative ai nodi
  		LogicTuple t2;
  		for(;;){
 			t2 =cn.in(tid, new LogicTuple("org_event",new Var("X"), new Var("Y")));
 			//aggiungo o elimino i nodi cancellando eventuali apici dai nomi
			if (t2.getArg(1).toString().equalsIgnoreCase("new_node")) myUniverse.addHost(t2.getArg(0).toString());
			if (t2.getArg(1).toString().equalsIgnoreCase("closed_node"))	myUniverse.deleteHost(t2.getArg(0).toString()); 
		
		}
	} catch (TucsonException e) {
		System.out.println("Probeli con il tucson");
	} catch (LogicTupleException e1) {
		System.out.println("Probeli con le tuple");
	}

	}
}
